unit KOLBAPTrayIcon;

interface

uses Windows, Messages, KOL;

////////////////////////////////////////////////////////////////////////////////
const
  { Define user-defined message sent by the trayicon. We avoid low user-defined
    messages that are used by Windows itself (eg. WM_USER+1 = DM_SETDEFID). }

  WM_TRAYNOTIFY         = WM_USER + 1024;

  // Constants used for balloon hint feature
  _NIIF_NONE            = $00000000;
  _NIIF_INFO            = $00000001;
  _NIIF_WARNING         = $00000002;
  _NIIF_ERROR           = $00000003;
  _NIIF_ICON_MASK       = $0000000F; // Reserved for WinXP
  _NIIF_NOSOUND         = $00000010; // Reserved for WinXP

  // Events returned by balloon hint
  _NIN_BALLOONSHOW      = WM_USER + 2;
  _NIN_BALLOONHIDE      = WM_USER + 3;
  _NIN_BALLOONTIMEOUT   = WM_USER + 4;
  _NIN_BALLOONUSERCLICK = WM_USER + 5;

  // Additional uFlags constants for TNotifyIconDataEx
  _NIF_STATE            = $00000008;
  _NIF_INFO             = $00000010;
  _NIF_GUID             = $00000020;

  // Additional dwMessage constants for Shell_NotifyIcon
  _NIM_SETFOCUS         = $00000003;
  _NIM_SETVERSION       = $00000004;

type
  { You can use the TNotifyIconData record structure defined in shellapi.pas.
    However, WinME, Win2000, and WinXP have expanded this structure, so in
    order to implement their new features we define a similar structure,
    TNotifyIconDataEx. }

  { The old TNotifyIconData record contains a field called Wnd in Delphi
    and hWnd in C++ Builder. The compiler directive DFS_CPPB_3_UP was used
    to distinguish between the two situations, but is no longer necessary
    when we define our own record, TNotifyIconDataEx. }

  TNotifyIconDataEx = record
    cbSize: DWORD;
    hWnd: HWND;
    uID: UINT;
    uFlags: UINT;
    uCallbackMessage: UINT;
    hIcon: HICON;
    szTip: array[0..127] of AnsiChar;  // Previously 64 chars, now 128
    dwState: DWORD;
    dwStateMask: DWORD;
    szInfo: array[0..255] of AnsiChar;
    case Integer of         // 0: Before Win2000; 1: Win2000 and up
      0: (uTimeout: UINT;
          szInfoTitle: array[0..63] of AnsiChar;
          dwInfoFlags: DWORD);
      1: (uVersion: UINT;   // Only used when sending a NIM_SETVERSION message
{$IFDEF _WIN32_IE_600)}
          guidItem: TGUID;  // Reserved for WinXP; define _WIN32_IE_600 if needed
{$ENDIF}
         )
  end;
////////////////////////////////////////////////////////////////////////////////

type

  PBAPTrayIcon = ^TBAPTrayIcon;
  TBAPTrayIcon = object(TObj)
  private
    NIDE: TNotifyIconDataEx;
    fControl: PControl;
    fIcon: HIcon;
    fToolTip: String;
    fBalloonText: String;
    fBalloonTitle: String;
    fOnBalloonShow: TOnEvent;
    fOnBalloonHide: TOnEvent;
    fOnBalloonClick: TOnEvent;
    fOnBalloonTimeOut: TOnEvent;
    fOnMouseMove: TOnMouse;
    fOnMouseDown: TOnMouse;
    fOnMouseUp: TOnMouse;
    fOnMouseDblClk: TOnMouse;
    procedure SetToolTip(Value: String);

  public
    property Icon: HIcon read fIcon write fIcon;
    property ToolTip: String read fToolTip write SetToolTip;
    property BalloonText: String read fBalloonText write fBalloonText;
    property BalloonTitle: String read fBalloonTitle write fBalloonTitle;
    property OnBalloonShow: TOnEvent read fOnBalloonShow write fOnBalloonShow;
    property OnBalloonHide: TOnEvent read fOnBalloonHide write fOnBalloonHide;
    property OnBalloonClick: TOnEvent read fOnBalloonClick write fOnBalloonClick;
    property OnBalloonTimeOut: TOnEvent read fOnBalloonTimeOut write fOnBalloonTimeOut;
    property OnMouseMove: TOnMouse read fOnMouseMove write fOnMouseMove;
    property OnMouseDown: TOnMouse read fOnMouseDown write fOnMouseDown;
    property OnMouseUp: TOnMouse read fOnMouseUp write fOnMouseUp;
    property OnMouseDblClk: TOnMouse read fOnMouseDblClk write fOnMouseDblClk;
    procedure AddIcon;
    procedure ModifyIcon;
    procedure DeleteIcon;
    procedure ShowBalloon(const IconType: Word);
    procedure HideBalloon;
    destructor Destroy; virtual;
  end;

  TKOLBAPTrayIcon = PBAPTrayIcon;

function NewBAPTrayIcon(Sender: PControl): PBAPTrayIcon;

implementation

uses ShellApi;

{--------------------}
{ Destructor  }
{--------------------}
destructor TBAPTrayIcon.Destroy;
begin
  fToolTip := '';
  DeleteIcon;
  DestroyIcon(fIcon);
    inherited;
end;

{--------------------}
{   }
{--------------------}
function WndProcTray(Ctl: PControl; var Msg: TMsg; var Rslt: Integer): Boolean;
var
  D: PBAPTrayIcon;
  CurXY: TPoint;
  Mouse: TMouseEventData;
begin
  D := Pointer(Msg.wParam);

  FillChar(Mouse, SizeOf(Mouse), 0);
  GetCursorPos(CurXY);
  Mouse.X := CurXY.x;
  Mouse.Y := CurXY.y;

  {   TrayIcon }
  if Msg.message = WM_TRAYNOTIFY then
    case Msg.lParam of
      _NIN_BALLOONSHOW: if Assigned(D.fOnBalloonShow) then D.fOnBalloonShow(Ctl);
      _NIN_BALLOONHIDE: if Assigned(D.fOnBalloonHide) then D.fOnBalloonHide(Ctl);
      _NIN_BALLOONUSERCLICK: if Assigned(D.fOnBalloonClick) then D.fOnBalloonClick(Ctl);
      _NIN_BALLOONTIMEOUT: if Assigned(D.fOnBalloonTimeOut) then D.fOnBalloonTimeOut(Ctl);
    end;{ case Msg.lParam }

  {    }
  case Msg.lParam of
    WM_MOUSEMOVE: if Assigned(D.fOnMouseMove) then D.fOnMouseMove(Ctl, Mouse);

    { OnMouseDown }

    WM_LBUTTONDOWN:
      if Assigned(D.fOnMouseDown) then
      begin
        Mouse.Button := mbLeft;
        D.fOnMouseDown(Ctl, Mouse);
      end;

    WM_RBUTTONDOWN:
      if Assigned(D.fOnMouseDown) then
      begin
        Mouse.Button := mbRight;
        D.fOnMouseDown(Ctl, Mouse);
      end;

    WM_MBUTTONDOWN:
      if Assigned(D.fOnMouseDown) then
      begin
        Mouse.Button := mbMiddle;
        D.fOnMouseDown(Ctl, Mouse);
      end;

    { OnMouseUp }

    WM_LBUTTONUP:
      if Assigned(D.fOnMouseUp) then
      begin
        Mouse.Button := mbLeft;
        D.fOnMouseUp(Ctl, Mouse);
      end;

    WM_RBUTTONUP:
      if Assigned(D.fOnMouseUp) then
      begin
        Mouse.Button := mbRight;
        D.fOnMouseUp(Ctl, Mouse);
      end;

    WM_MBUTTONUP:
      if Assigned(D.fOnMouseUp) then
      begin
        Mouse.Button := mbMiddle;
        D.fOnMouseUp(Ctl, Mouse);
      end;

    { OnMouseDblClk }

    WM_LBUTTONDBLCLK:
      if Assigned(D.fOnMouseDblClk) then
      begin
        Mouse.Button := mbLeft;
        D.fOnMouseDblClk(Ctl, Mouse);
      end;

    WM_RBUTTONDBLCLK:
      if Assigned(D.fOnMouseDblClk) then
      begin
        Mouse.Button := mbRight;
        D.fOnMouseDblClk(Ctl, Mouse);
      end;

    WM_MBUTTONDBLCLK:
      if Assigned(D.fOnMouseDblClk) then
      begin
        Mouse.Button := mbMiddle;
        D.fOnMouseDblClk(Ctl, Mouse);
      end;
  end; { case Msg.LParam }

  Result := Mouse.StopHandling;
end;
////////////////////////////////////////////////////////////////////////////////

{-----------------------------}
{   KOL  }
{-----------------------------}
function NewBAPTrayIcon;
begin
  New(Result, Create);
  Result.fControl := Sender;

  {    PControl }
  Sender.AttachProc(WndProcTray);
end;
////////////////////////////////////////////////////////////////////////////////

{-----------------}
{   }
{-----------------}
procedure TBAPTrayIcon.AddIcon;
begin
  FillChar(NIDE, SizeOf(NIDE), 0);

  with NIDE do
  begin
    cbSize := SizeOf(NIDE);
    uID := DWORD(@Self); {   (    ) }
    hIcon := fIcon;      { Handle  }
    hWnd := fControl.Handle;
    uFlags := NIF_MESSAGE or NIF_ICON or NIF_TIP;
    uCallbackMessage := WM_TRAYNOTIFY; {   }
    lstrcpy(szTip, PChar(ToolTip));    {    }
  end;

 Shell_NotifyIcon(NIM_ADD, @NIDE);
end;

{-----------------}
{   }
{-----------------}
procedure TBAPTrayIcon.ModifyIcon;
begin
  NIDE.hIcon := fIcon; { Handle  }
  NIDE.uFlags := NIF_ICON;  {  }
  Shell_NotifyIcon(NIM_MODIFY, @NIDE);
end;

{------------------------------}
{     }
{------------------------------}
procedure TBAPTrayIcon.SetToolTip;
begin
  fToolTip := Value;
  NIDE.uFlags := NIF_TIP; {  }
  lstrcpy(NIDE.szTip, PChar(fToolTip));
  Shell_NotifyIcon(NIM_MODIFY, @NIDE);
end;

{----------------}
{   }
{----------------}
procedure TBAPTrayIcon.DeleteIcon;
begin
  Shell_NotifyIcon(NIM_DELETE, @NIDE);
end;
////////////////////////////////////////////////////////////////////////////////

{------------------}
{  Balloon }
{------------------}
procedure TBAPTrayIcon.ShowBalloon;
begin
  with NIDE do
  begin
    uFlags := _NIF_INFO;
    dwInfoFlags := IconType;
    lstrcpy(szInfoTitle, PChar(fBalloonTitle)); {  Balloon` }
    lstrcpy(szInfo, PChar(fBalloonText));       {  Balloon` }
    Shell_NotifyIcon(NIM_MODIFY, @NIDE);
    szInfoTitle[0] := #0;
    szInfo[0] := #0;
  end; { with }
end;

{------------------}
{  Balloon }
{------------------}
procedure TBAPTrayIcon.HideBalloon;
begin
  Shell_NotifyIcon(NIM_MODIFY, @NIDE);
end;
////////////////////////////////////////////////////////////////////////////////

end.